void runGame() {
  background(255);

  //Draw track
  update();  //Update physics positions
  drawTrack(); //Draw track 
  drawStats();  //Draw spped, time, etc.
  drawCar();  //Draw the car oriented right
  sound();  //Adjust sounds
}

//Updates the cars position
void update() {
  //Calculate new x, y
  calcSpeed();
  timerControl(); //Manipulate timer as needed
  //Move positions
  float x = -(speed * cos(rotation));  //Reversed as relative to car
  float y = -(speed * sin(rotation)); 
  posX += x;
  posY += y;
}

void drawTrack() {
  pushMatrix();
  imageMode(CORNER);
  translate(width/2, height/2);
  image (map, posX, posY, map.width * trackScale, map.height * trackScale);
  popMatrix();
}

void drawCar() {  
  //Draw car
  //Rotate
  pushMatrix();
  imageMode(CENTER);
  translate(width/2, height/2);
  rotate(PI/2 + rotation);  //Rotate from facing up to origin plus the angle of the car 
  image(teams[teamSelector].getCarImg(), 0, 0, teams[teamSelector].getCarImg().width * carScale, teams[teamSelector].getCarImg().height * carScale);  //Draw car
  popMatrix();
}

//Calc the speed of the car based on keyboard inputs
void calcSpeed() {
  //Check to see if on track or not
  boolean onTrack = false;
  color positionColor = map.get(int(-1*posX), int(-1*posY)); //Get the color at the cars position
  if (positionColor == color(132, 132, 132) || positionColor == color(205, 45, 30) || positionColor == color(255, 255, 255) || positionColor == color(0, 0, 0)) {
    onTrack = true;
  }

  //Accelerating
  if (forward && !brake) {
    //See if on track gray, red, or white
    if (onTrack) {
      if (speed < maxSpeed) {
        speed += acceleration;
      }
    } else {
      if (speed < sandMaxSpeed) {
        speed += sandAcceleration;
      } else if (speed > sandMaxSpeed) {
        speed -= sandDeacceleration;
      }
    }
  }

  //Braking
  else if (brake && !forward) {
    if (onTrack) {
      speed -= brakeDeacceleration;
    } else {
      speed -= sandDeacceleration;
    }
  }
  //If coasting
  else if (forward && brake || !brake && !forward) {
    if (onTrack) {
      speed -= coastDeacceleration;
    } else {
      speed -= sandDeacceleration;
    }
  }

  //Check to see if speed is negative and not in reverse
  if (!reverse) {
    if (speed < 0) {
      speed = 0;
    }
  }

  //Check to see if the car crashed
  if (positionColor == color(19, 30, 255)) {
    speed = 0;
    gameMode = 3;
  }

  //Check right and left
  if (left && !right && speed != 0) {
    //Check if in reverse
    if (speed > 0) {
      rotation -= angularAcceleration;
    } else {
      rotation += angularAcceleration;
    }
  } else if (right && !left && speed != 0) {
    //Check if in reverse
    if (speed > 0) {
      rotation += angularAcceleration;
    } else {
      rotation -= angularAcceleration;
    }
  }
}

void drawStats() {
  //Speed triangle
  fill(0);
  beginShape();
  vertex(width, 0);
  vertex(width, height/6);
  vertex(width - height/6, 0);
  endShape();

  //Speed
  textAlign(CENTER);
  fill(255);
  textSize(15);
  text("Speed: " + 2*int(speed) + "km/h", width - height/16, height/24);

  //Current time rect
  fill(0);
  rectMode(CENTER);
  rect(width/2, 1.4*(height/24), width/7, height/13);

  //Current Time
  fill(255);
  passedTime = (millis() - savedTime) / 1000;
  textSize(20);
  text("Current Lap: " + int(passedTime/60) + "m " + passedTime%60 + "s", width/2, height/24);

  //Best Time
  fill(255);
  if(bestTime != -1){
    text("Best Lap: " + int(bestTime/60) + "m " + bestTime%60 + "s", width/2, 2*(height/24));
  }
  else{
    text("Best Lap: " + "none", width/2, 2*(height/24));
  }
  
  //Return to menu button
  fill(0);
  beginShape();
  vertex(0, 0);
  vertex(0, height/6);
  vertex(height/6, 0);
  endShape();
  
  //Return to menu text
  textAlign(CENTER);
  fill(255);
  textSize(15);
  text("Press BACKSPACE",height/16, height/24);
  text("to exit",height/16, height/24 + 20);
  
}

void timerControl() {
  //println(posX + " " + posY);
  //If crossed start line going forward
  if(((-1*posY) < 2110) && ((-1*posX) > 4714) && ((-1*(posX - (speed * cos(rotation)))) < 4714)){
    //If there was an active lap running, save best time if it is less or if no previous best time
    if(activeTimer && (((millis() - savedTime) / 1000 < bestTime) || bestTime == -1)){
      bestTime = (millis() - savedTime) / 1000;
    }
    activeTimer = true; //Start a active timer
    savedTime = millis(); //Start timer
  }
  //If crossed going backwards
  else if(((-1*posY) < 2110) && ((-1*posX) < 4714) && ((-1*(posX - (speed * cos(rotation)))) > 4714)){
    activeTimer = false;
  }
}
